function [uhat, xhat, numIters] = SolveTVL1CoeffsMultDouglasRachford(A, y, n, p, tvoptions, lambda, gamma, tightFrame, maxIters, fullPath, verbose)
% SolveTVL1CoeffsMultDouglasRachford: Iterative Douglas-Rachford Splitting Proximal iteration to solve the l1-fidelity and TV minimization problem:
%		min ||Lambda(y - x)||_1 + || A^+ x ||_TV , Lambda is a diagonal weight matrix.
%
%	NOTE: The iteration below is rigorously valid for tight frames A with frame constant c such that A^+A'=Id, A^+=c^-1 A.
%
%	The solution is given by the iteration :
%		x^(t+1/2) = (y + ST_{gamma*lambda}[x^(t) - y])
%		x^(t+1)   = x^(t) + mu_t (prox_gamma|| A^+ . ||_TV[2 x^(t+1/2) - x^(t)] - x^(t+1/2)),
%	where 
%		ST_{gamma*lambda}: is the component-wise soft-thresholding operator with thresholds (gamma*lambda_k)_k=1,...,p.
%		prox_gamma|| A . ||_TV[x] = x + c^-1 A'(prox_TV - Id)A(x) .
%		
%		0 < mu_t < 2, fixed to mu_t=1 in this code (optimal convergence speed).
%		
%		prox_TV[x] is the proximal operator of gamma*||.||_TV (computed through Legendre-Fenchel duality and Moreau decomposition
%			   In 1D, it is implemented by a forward-backward iteration. In 2D, fast algorithm of CD).
%
%
%	The solution to the minimization problem is given by x^(t+1/2) at convergence.
%	The image estimate can be reconstructed for the solution to this minimization problem as u = A^+ x^(t+1/2) at convergence.
%
% Usage
%	[xhat, uhat, numIters] = SolveTVL1CoeffsMultDouglasRachford(A, y, p, tvoptions, lambda, gamma, tightFrame, maxIters, fullPath, verbose, positivity)
% Input
%	A           nxp matrix or implicit operator
%	y           p-vector of thresholded coefficients
%	n	    Length of image
%  	p           length of coefficients
%	tvoptions   a structure containing options for TV prox subiterations:
%			name		name of the algorithm that solves for the prox 
%					forward-backward, Chambolle, fast Chambolle-Darbon (for 2D).
%			dimension	1 for 1D vectors and 2 for 2D images (sqrt(p) x sqrt(p) images)
%			numdeep		depth of the dyadic search for the fast TV prox (only for 2D) (optional)
%			lmin,lmax	the solution is truncated to lmin/lmax (optional)
%			mu		relaxation parameter for the forward-backward implementation case (optional)
%			iterstv		number of iterations for the 1D case (optional)
%	lambda	    a vector of 2 regularization parameters, first thresholded coeffs and second those kept
%   	gamma	    a strictly positive real, default 1
%	tightFrame  constant of the tight frame if any.
%	maxIters
%   	fullPath    1 returns entire solution path, 0 final
%   	verbose     
% Outputs
%	 uhat       image reconstructed from solution coefficients
%	 xhat       solution coefficients of the problem
%	 numIters
%
global dict

fastop = (ischar(A) || isa(A, 'function_handle'));

if ~exist('verbose'), 	 verbose = 0; 		end
if ~exist('fullPath'), 	 fullPath = 0; 		end
if ~exist('maxIters'), 	 maxIters = 50; 	end
if ~exist('tightFrame'), tightFrame = 1;	end
if ~exist('gamma'), 	 gamma = 1;		end
if ~exist('lambda'), 	 lambda = 1;		end
if ~exist('p'), 	 p = size(A,2); 	end
if ~exist('n'), 	 n = size(A,1); 	end
if ~exist('tvoptions'),  
	tvoptions.dimension = '1';
	tvoptions.name = 'perform_prox_tv';
	tvoptions.mu = 0.4;
	tvoptions.iterstv = 200;
	tvoptions.tvtol = 1E-3;
end

% lsqrms params
damp = 0;
atol = 1e-6;
btol = 1e-6;
conlim = 1e+10;
itnlim = 10;
show = 0;


tmp = lambda;
lambda = tmp(1)*ones(size(y));	% Thresholded coeffs.
lambda(y~=0) = tmp(2);	 	% Kept coeffs.
xhat = zeros(p,1);
Ifull = 1:p;
iter = 1;
sols = [];
xhat1 = 0;
if tvoptions.dimension == '1'
   if ~isfield(tvoptions,'name'), 	tvoptions.name = 'perform_prox_tv'; end
   if ~isfield(tvoptions,'mu'), 	tvoptions.mu = 0.4;  end
   if ~isfield(tvoptions,'iterstv'), 	tvoptions.iterstv = 200; end
   if ~isfield(tvoptions,'tvtol'), 	tvoptions.tvtol = 1E-3;  end
   if ~isfield(tvoptions,'verbose'), 	tvoptions.verbose = 0;   end
else
   if ~isfield(tvoptions,'name'), 	tvoptions.name = 'perform_chambollefast_tv'; end
   if ~isfield(tvoptions,'numdeep'), 	tvoptions.numdeep = 8; 	    end

% Options for old forward-backward code.
% if ~isfield(tvoptions,'mu'),	       tvoptions.mu = 0.25;      end
% if ~isfield(tvoptions,'iterstv'),    tvoptions.iterstv = 400; end
% if ~isfield(tvoptions,'tvtol'),      tvoptions.tvtol = 1E-3;  end
end



%while  (iter <= maxIters) & (norm(res) > OptTol*normy)

while  (iter <= maxIters)
    %xhat1 = y + SoftThresh(xhat-y,gamma*lambda/tightFrame);
    %xdiff = 2*xhat1 - xhat;
    if fastop
        xrec = feval(A,1,n,p,xhat,Ifull,p)/tightFrame;
	if isreal(xhat(1)) 
		%tvoptions.lmin = min(xrec(:));
		%tvoptions.lmax = max(xrec(:));
		proxtv = eval([tvoptions.name tvoptions.dimension 'D(xrec,gamma/tightFrame,tvoptions)']);
        else
        	disp('The complex case is not implemented yet for TV minimization.');
    		return
    	end
	corr = feval(A,2,n,p,proxtv-xrec,Ifull,p);
    else
    	xrec = A*xhat/tightFrame;
	if isreal(xhat(1)) 
		%tvoptions.lmin = min(xrec(:));
		%tvoptions.lmax = max(xrec(:));
    		proxtv = eval([tvoptions.name tvoptions.dimension 'D(xrec/tightFrame,gamma,tvoptions)']);
        else
        	disp('The complex case is not implemented yet for TV minimization.');
    		return
    	end
        corr = A'*(proxtv-xrec);
    end
    xold  = xhat;
    xhat1 = xhat + corr;
    xdiff = 2*xhat1 - xhat;
    xhat = y + SoftThresh(xdiff-y,gamma*lambda) - corr;
    %xhat = xhat1 + corr;
      
    if (verbose)
    	if fastop
    	   uhat = feval(A,1,n,p,xhat1,Ifull,p)/tightFrame;
	else
	   uhat = A*xhat1/tightFrame;
	end
	subplot(121);
	imagesc(reshape(exp(uhat),floor(sqrt(n)),floor(sqrt(n))));axis image;rmaxis;colorbar;drawnow
    	tvnorm(iter) = sum(sum(sqrt(sum(perform_grad(uhat,1).^2,1))));
        %fprintf('Iteration %d: ||u||_TV = %g\n', iter, tvnorm(iter));
	rfid(iter) = norm(lambda.*(y - xhat1)/tightFrame,1);
	ruhat(iter) = norm(xhat-xold);
	subplot(222)
	plot([rfid' tvnorm' rfid'+tvnorm']);legend('lambda||residual||_1','||.||_{TV}','Objective');drawnow
	subplot(224)
	plot(ruhat);legend('||x^{n+1}-x^{n}||_2');drawnow
    end

    if fullPath
        sols = [sols xhat1];
    end

    iter = iter+1;
end

xhat = xhat1;

if fastop
   uhat = feval(A,1,n,p,xhat,Ifull,p)/tightFrame;
else
   uhat = A*xhat/tightFrame;
end

if fullPath
    xhat = sols;
end

numIters = iter;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ud = perform_grad(u,h)
%  Computes the forward finite differences, ie.
%	    ud(1,i,j) = (u(i+1,j)-u(i,j))/(h),
%	    ud(2,i,j) = (u(i,j+1)-u(i,j))/(h), with special
%           care at boundaries (ud(n) = 0).
%	    returns |ud|.
%

[n,m] = size(u);
ud = zeros(2,n,m);
ud(1,:,:) = [diff(u,1,1);zeros(1,m)]/h;
ud(2,:,:) = [diff(u,1,2),zeros(n,1)]/h;

%
% Part of CurvL1MultDenoise Version:100
% Written by: Jalal Fadili, GREYC CNRS-ENSICAEN-Univ. Caen
%      	      Mila Nikolova, CMLA CNRS-ENS Cachan
%	      Sylvain Durand, MAP5 CNRS-Univ. Paris 5
% Created March 2009
% This is Copyrighted Material
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
%
% Modified Wed Dec 16 18:41:37 CET 2009
%

